@@ -33,6 +33,7 @@
 #define ROBOTS_OPTIONS		0x0008000
 
 typedef struct robots_entry {
+  int rowid;
   int info;
 
   char netloc[WEBBASE_URL_LENGTH + 1];
@@ -62,6 +63,8 @@ typedef struct robots {
 struct option* robots_options(struct option options[]);
 robots_t* robots_alloc(int argc, char** argv, struct option options[]);
 void robots_load(robots_t* robots, crawl_context_t* context, char* url);
+int robots_server_id(robots_t* robots);
+int robots_url2server_id(webbase_t* base, char* url);
 void robots_info(robots_t* robots, int robot_delay, uri_t* url_object, time_t* delay, int* delayed);
 void robots_free(robots_t* params);
 int robots_p(char* path);
