@@ -24,6 +24,12 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <string.h>
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif /* HAVE_MALLOC_H */
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif /* HAVE_STDLIB_H */
 #ifdef HAVE_DMALLOC_H
 #include <dmalloc.h>
 #endif /* HAVE_DMALLOC_H */
@@ -36,26 +42,40 @@
 #include <file_size.h>
 
 #include <webbase_url.h>
-#include <hrefparse.h>
+#include <html.h>
 
 static webbase_url_t* _webbase_url_object = 0;
-static int _webbase_url_object_size = 0;
 
 static int verbose = 0;
 
 webbase_url_t* webbase_url_object()
 {
-  static_alloc((char**)&_webbase_url_object, &_webbase_url_object_size, sizeof(webbase_url_t));
-  memset((char*)_webbase_url_object, '\0', sizeof(webbase_url_t));
+  if(_webbase_url_object == 0) {
+    _webbase_url_object = (webbase_url_t*)smalloc(sizeof(webbase_url_t));
+    memset((char*)_webbase_url_object, '\0', sizeof(webbase_url_t));
+  } else {
+    webbase_url_reset(_webbase_url_object, WEBBASE_URL_RESET_TOTAL);
+  }
   return _webbase_url_object;
 }
 
+void webbase_url_free(webbase_url_t* webbase_url)
+{
+  if(webbase_url->w_url) free(webbase_url->w_url);
+  if(webbase_url->w_base_url) free(webbase_url->w_base_url);
+  if(webbase_url->w_relative) free(webbase_url->w_relative);
+  if(webbase_url->w_absolute) free(webbase_url->w_absolute);
+  if(webbase_url->w_location) free(webbase_url->w_location);
+}
+
 void webbase_copy_url(webbase_url_t* to, webbase_url_t* from)
 {
   webbase_url_reset(to, WEBBASE_URL_RESET_TOTAL);
   
   to->w_rowid = from->w_rowid;
+  static_alloc(&to->w_url, &to->w_url_length, from->w_url_length);
   strcpy(to->w_url, from->w_url);
+  memcpy(to->w_url_md5, from->w_url_md5, MD5_ASCII_SIZE);
   to->w_info = from->w_info;
   to->w_code = from->w_code;
   to->w_mtime = from->w_mtime;
@@ -63,7 +83,7 @@ void webbase_copy_url(webbase_url_t* to, webbase_url_t* from)
   to->w_tags = from->w_tags;
   strcpy(to->w_content_type, from->w_content_type);
   to->w_content_length = from->w_content_length;
-  memcpy(to->w_md5, from->w_md5, WEBBASE_MD5_LENGTH);
+  memcpy(to->w_md5, from->w_md5, MD5_ASCII_SIZE);
   to->w_complete_rowid = from->w_complete_rowid;
   to->w_crawl = from->w_crawl;
   to->w_hookid = from->w_hookid;
@@ -97,7 +117,10 @@ void webbase_copy_url(webbase_url_t* to, webbase_url_t* from)
 void webbase_url_reset(webbase_url_t* webbase_url, int flag)
 {
   if(flag != WEBBASE_URL_RESET_PARTIAL) webbase_url->w_rowid = 0;
-  if(flag != WEBBASE_URL_RESET_PARTIAL) webbase_url->w_url[0] = 0;
+  if(flag != WEBBASE_URL_RESET_PARTIAL) {
+    if(webbase_url->w_url) webbase_url->w_url[0] = '\0';
+    memset(webbase_url->w_url_md5, '\0', MD5_ASCII_SIZE);
+  }
   webbase_url->path = 0;
   webbase_url->w_info = 0;
   webbase_url->w_code = 0;
@@ -106,7 +129,7 @@ void webbase_url_reset(webbase_url_t* webbase_url, int flag)
   if(flag != WEBBASE_URL_RESET_PARTIAL) webbase_url->w_tags = 0;
   webbase_url->w_content_type[0] = '\0';
   webbase_url->w_content_length = 0;
-  webbase_url->w_md5[0] = '\0';
+  memset(webbase_url->w_md5, '\0', MD5_ASCII_SIZE);
   if(flag != WEBBASE_URL_RESET_PARTIAL) webbase_url->w_complete_rowid = 0;
   webbase_url->w_crawl = 0;
   if(flag != WEBBASE_URL_RESET_PARTIAL) webbase_url->w_hookid = 0;
@@ -115,13 +138,9 @@ void webbase_url_reset(webbase_url_t* webbase_url, int flag)
   webbase_url->w_title[0] = '\0';
   webbase_url->w_keywords[0] = '\0';
   webbase_url->w_description[0] = '\0';
-  webbase_url->w_base_url_length = 0;
   if(webbase_url->w_base_url) webbase_url->w_base_url[0] = '\0';
-  webbase_url->w_relative_length = 0;
   if(webbase_url->w_relative) webbase_url->w_relative[0] = '\0';
-  webbase_url->w_absolute_length = 0;
   if(webbase_url->w_absolute) webbase_url->w_absolute[0] = '\0';
-  webbase_url->w_location_length = 0;
   if(webbase_url->w_location) webbase_url->w_location[0] = '\0';
 }
 
@@ -172,6 +191,24 @@ int webbase_url_walk_href(webbase_url_t* webbase_url, int flag, webbase_url_walk
   return !end;
 }
 
+/*
+ * Return 1 if url is suitable for crawl starting point, 0 otherwise.
+ */
+int webbase_url_start_ok(char* url, int url_length)
+{
+  uri_t* uri = uri_object(url, url_length);
+  if(!uri) return 0;
+  if(uri_info(uri) & URI_INFO_RELATIVE) {
+    fprintf(stderr, "webbase_url_start_ok: a starting point cannot be a relative url\n");
+    return 0;
+  }
+  if(strncmp(url, uri_uri(uri), url_length)) {
+    fprintf(stderr, "webbase_url_start_ok: a starting point must be cannonical\n");
+    return 0;
+  }
+  return 1;
+}
+
 void webbase_url_print(webbase_url_t* webbase_url)
 {
 #if 0
@@ -374,5 +411,5 @@ void webbase_url_mtime_fix(webbase_url_t* webbase_url)
 
 void webbase_url_hrefs_set(webbase_url_t* webbase_url, char* path, int size_limit)
 {
-  hrefparse(path, HREFPARSE_SOURCE_FILENAME, HREFPARSE_KEEP_RELATIVE, webbase_url, size_limit);
+  html_href(path, webbase_url, size_limit);
 }
