@@ -22,6 +22,6 @@
 extern unsigned char iso2letter_unaccent[256];
 extern unsigned char iso2letter_transparent[256];
 
-unsigned char* unaccent(unsigned char* string);
+unsigned char* unaccent(unsigned char* string, int string_length);
 
 #endif /* isomap_h */
